/***************************************************************************
                          ksnumbers.cpp  -  description
                             -------------------
    begin                : Sun Jan 13 2002
    copyright            : (C) 2002 by Jason Harris
    email                : kstars@30doradus.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ksnumbers.h"

// 63 elements
const int KSNumbers::arguments[NUTTERMS][5] = {
{ 0, 0, 0, 0, 1},
{-2, 0, 0, 2, 2},
{ 0, 0, 0, 2, 2},
{ 0, 0, 0, 0, 2},
{ 0, 1, 0, 0, 0},
{ 0, 0, 1, 0, 0},
{-2, 1, 0, 2, 2},
{ 0, 0, 0, 2, 1},
{ 0, 0, 1, 2, 2},
{-2,-1, 0, 2, 2},
{-2, 0, 1, 0, 0},
{-2, 0, 0, 2, 1},
{ 0, 0,-1, 2, 2},
{ 2, 0, 0, 0, 0},
{ 0, 0, 1, 0, 1},
{ 2, 0,-1, 2, 2},
{ 0, 0,-1, 0, 1},
{ 0, 0, 1, 2, 1},
{-2, 0, 2, 0, 0},
{ 0, 0,-2, 2, 1},
{ 2, 0, 0, 2, 2},
{ 0, 0, 2, 2, 2},
{ 0, 0, 2, 0, 0},
{-2, 0, 1, 2, 2},
{ 0, 0, 0, 2, 0},
{-2, 0, 0, 2, 0},
{ 0, 0,-1, 2, 1},
{ 0, 2, 0, 0, 0},
{ 2, 0,-1, 0, 1},
{-2, 2, 0, 2, 2},
{ 0, 1, 0, 0, 1},
{-2, 0, 1, 0, 1},
{ 0,-1, 0, 0, 1},
{ 0, 0, 2,-2, 0},
{ 2, 0,-1, 2, 1},
{ 2, 0, 1, 2, 2},
{ 0, 1, 0, 2, 2},
{-2, 1, 1, 0, 0},
{ 0,-1, 0, 2, 2},
{ 2, 0, 0, 2, 1},
{ 2, 0, 1, 0, 0},
{-2, 0, 2, 2, 2},
{-2, 0, 1, 2, 1},
{ 2, 0,-2, 0, 1},
{ 2, 0, 0, 0, 1},
{ 0,-1, 1, 0, 0},
{-2,-1, 0, 2, 1},
{-2, 0, 0, 0, 1},
{ 0, 0, 2, 2, 1},
{-2, 0, 2, 0, 1},
{-2, 1, 0, 2, 1},
{ 0, 0, 1,-2, 0},
{-1, 0, 1, 0, 0},
{-2, 1, 0, 0, 0},
{ 1, 0, 0, 0, 0},
{ 0, 0, 1, 2, 0},
{ 0, 0,-2, 2, 2},
{-1,-1, 1, 0, 0},
{ 0, 1, 1, 0, 0},
{ 0,-1, 1, 2, 2},
{ 2,-1,-1, 2, 2},
{ 0, 0, 3, 2, 2},
{ 2,-1, 0, 2, 2}
};

const int KSNumbers::amp[NUTTERMS][4] = {
{-171996,-1742, 92025, 89},
{ -13187,  -16,  5736,-31},
{  -2274,   -2,   977, -5},
{   2062,    2,  -895,  5},
{   1426,  -34,    54, -1},
{    712,    1,    -7,  0},
{   -517,   12,   224, -6},
{   -386,   -4,   200,  0},
{   -301,    0,   129, -1},
{    217,   -5,   -95,  3},
{   -158,    0,     0,  0},
{    129,    1,   -70,  0},
{    123,    0,   -53,  0},
{     63,    0,     0,  0},
{     63,    1,   -33,  0},
{    -59,    0,    26,  0},
{    -58,   -1,    32,  0},
{    -51,    0,    27,  0},
{     48,    0,     0,  0},
{     46,    0,   -24,  0},
{    -38,    0,    16,  0},
{    -31,    0,    13,  0},
{     29,    0,     0,  0},
{     29,    0,   -12,  0},
{     26,    0,     0,  0},
{    -22,    0,     0,  0},
{     21,    0,   -10,  0},
{     17,   -1,     0,  0},
{     16,    0,    -8,  0},
{    -16,    1,     7,  0},
{    -15,    0,     9,  0},
{    -13,    0,     7,  0},
{    -12,    0,     6,  0},
{     11,    0,     0,  0},
{    -10,    0,     5,  0},
{     -8,    0,     3,  0},
{      7,    0,    -3,  0},
{     -7,    0,     0,  0},
{     -7,    0,     3,  0},
{     -7,    0,     3,  0},
{      6,    0,     0,  0},
{      6,    0,    -3,  0},
{      6,    0,    -3,  0},
{     -6,    0,     3,  0},
{     -6,    0,     3,  0},
{      5,    0,     0,  0},
{     -5,    0,     3,  0},
{     -5,    0,     3,  0},
{     -5,    0,     3,  0},
{      4,    0,     0,  0},
{      4,    0,     0,  0},
{      4,    0,     0,  0},
{     -4,    0,     0,  0},
{     -4,    0,     0,  0},
{     -4,    0,     0,  0},
{      3,    0,     0,  0},
{     -3,    0,     0,  0},
{     -3,    0,     0,  0},
{     -3,    0,     0,  0},
{     -3,    0,     0,  0},
{     -3,    0,     0,  0},
{     -3,    0,     0,  0},
{     -3,    0,     0,  0}
};


KSNumbers::KSNumbers( long double jd ){
	K.setD( 20.49552 / 3600. );  //set the constant of aberration
	updateValues( jd );
}

KSNumbers::~KSNumbers(){
}

void KSNumbers::updateValues( long double jd ) {
	dms arg;
	double args, argc;

	days = jd;

	//Julian Centuries since J2000.0
	T = ( jd - J2000 ) / 36525.;

	// Julian Millenia since J2000.0
	jm = T / 10.0;

	double T2 = T*T;
	double T3 = T2*T;

	//Sun's Mean Longitude
	L.setD( 280.46645 + 36000.76983*T + 0.0003032*T2 );

	//Mean elongation of the Moon from the Sun
	D.setD( 297.85036 + 445267.111480*T - 0.0019142*T2 + T3/189474.);
		
	//Sun's Mean Anomaly
	M.setD( 357.52910 + 35999.05030*T - 0.0001559*T2 - 0.00000048*T3);
	
	//Moon's Mean Anomaly
	MM.setD( 134.96298 + 477198.867398*T + 0.0086972*T2 + T3/56250.0 );

	//Moon's Mean Longitude
	LM.setD( 218.3164591 + 481267.88134236*T - 0.0013268*T2 + T3/538841. - T*T*T*T/6519400.);
	
	//Moon's argument of latitude
	F.setD( 93.27191 + 483202.017538*T - 0.0036825*T2 + T3/327270.);

	//Longitude of Moon's Ascending Node
	O.setD( 125.04452 - 1934.136261*T + 0.0020708*T2 + T3/450000.0 );

	//Earth's orbital eccentricity
	e = 0.016708617 - 0.000042037*T - 0.0000001236*T2;
	
	double C = ( 1.914600 - 0.004817*T - 0.000014*T2 ) * sin( M.radians() )
						+ ( 0.019993 - 0.000101*T ) * sin( 2.0* M.radians() )
						+ 0.000290 * sin( 3.0* M.radians() );

	//Sun's True Longitude
	L0.setD( L.Degrees() + C );

	//Sun's True Anomaly
	M0.setD( M.Degrees() + C );

	//Obliquity of the Ecliptic
	double U = T/100.0;
	double dObliq = -4680.93*U - 1.55*U*U + 1999.25*U*U*U
									- 51.38*U*U*U*U - 249.67*U*U*U*U*U
									- 39.05*U*U*U*U*U*U + 7.12*U*U*U*U*U*U*U
									+ 27.87*U*U*U*U*U*U*U*U + 5.79*U*U*U*U*U*U*U*U*U
									+ 2.45*U*U*U*U*U*U*U*U*U*U;
	Obliquity.setD( 23.43929111 + dObliq/3600.0);

	//Nutation parameters
	dms  L2, M2, O2;
	double sin2L, cos2L, sin2M, cos2M;
	double sinO, cosO, sin2O, cos2O;
	
	O2.setD( 2.0*O.Degrees() );
	L2.setD( 2.0*L.Degrees() ); //twice mean ecl. long. of Sun
	M2.setD( 2.0*LM.Degrees() );  //twice mean ecl. long. of Moon
	
	O.SinCos( sinO, cosO );
	O2.SinCos( sin2O, cos2O );
	L2.SinCos( sin2L, cos2L );
	M2.SinCos( sin2M, cos2M );
	
//	deltaEcLong = ( -17.2*sinO - 1.32*sin2L - 0.23*sin2M + 0.21*sin2O)/3600.0; //Ecl. long. correction
//	deltaObliquity = ( 9.2*cosO + 0.57*cos2L + 0.10*cos2M - 0.09*cos2O)/3600.0; //Obliq. correction

	deltaEcLong = 0.;
	deltaObliquity = 0.;
	
	for (unsigned int i=0; i < NUTTERMS; i++) {

		arg.setD ( arguments[i][0]*D.Degrees() + arguments[i][1]*M.Degrees() + 
			arguments[i][2]*MM.Degrees() + arguments[i][3]*F.Degrees() + arguments[i][4]*O.Degrees() );
		arg.SinCos( args, argc );

		deltaEcLong +=  (amp[i][0] + amp[i][1]/10. * T ) * args * 1e-4 ;
		deltaObliquity += (amp[i][2] + amp[i][3]/10. * T ) * argc * 1e-4 ;
	}

	deltaEcLong/= 3600.0;
	deltaObliquity /= 3600.0;

	//Compute Precession Matrices:
	XP.setD( 0.6406161*T + 0.0000839*T2 + 0.0000050*T3 );
	YP.setD( 0.5567530*T - 0.0001185*T2 - 0.0000116*T3 );
	ZP.setD( 0.6406161*T + 0.0003041*T2 + 0.0000051*T3 );

	XP.SinCos( SX, CX );
	YP.SinCos( SY, CY );
	ZP.SinCos( SZ, CZ );
	
//P1 is used to precess from any epoch to J2000
	P1[0][0] = CX*CY*CZ - SX*SZ;
	P1[1][0] = CX*CY*SZ + SX*CZ;
	P1[2][0] = CX*SY;
	P1[0][1] = -1.0*SX*CY*CZ - CX*SZ;
	P1[1][1] = -1.0*SX*CY*SZ + CX*CZ;
	P1[2][1] = -1.0*SX*SY;
	P1[0][2] = -1.0*SY*CZ;
	P1[1][2] = -1.0*SY*SZ;
	P1[2][2] = CY;

//P2 is used to precess from J2000 to any other epoch (it is the transpose of P1)
	P2[0][0] = CX*CY*CZ - SX*SZ;
	P2[1][0] = -1.0*SX*CY*CZ - CX*SZ;
	P2[2][0] = -1.0*SY*CZ;
	P2[0][1] = CX*CY*SZ + SX*CZ;
	P2[1][1] = -1.0*SX*CY*SZ + CX*CZ;
	P2[2][1] = -1.0*SY*SZ;
	P2[0][2] = CX*SY;
	P2[1][2] = -1.0*SX*SY;
	P2[2][2] = CY;

//Compute Precession Matrices from B1950 to 1984 using Newcomb formulae

	XB.setD( 0.217697 );
	YB.setD( 0.189274 );
	ZB.setD( 0.217722 );

	XB.SinCos( SXB, CXB );
	YB.SinCos( SYB, CYB );
	ZB.SinCos( SZB, CZB );

//P1B is used to precess from 1984 to B1950:

	P1B[0][0] = CXB*CYB*CZB - SXB*SZB;
	P1B[1][0] = CXB*CYB*SZB + SXB*CZB;
	P1B[2][0] = CXB*SYB;
	P1B[0][1] = -1.0*SXB*CYB*CZB - CXB*SZB;
	P1B[1][1] = -1.0*SXB*CYB*SZB + CXB*CZB;
	P1B[2][1] = -1.0*SXB*SYB;
	P1B[0][2] = -1.0*SYB*CZB;
	P1B[1][2] = -1.0*SYB*SZB;
	P1B[2][2] = CYB;

//P2 is used to precess from B1950 to 1984 (it is the transpose of P1)
	P2B[0][0] = CXB*CYB*CZB - SXB*SZB;
	P2B[1][0] = -1.0*SXB*CYB*CZB - CXB*SZB;
	P2B[2][0] = -1.0*SYB*CZB;
	P2B[0][1] = CXB*CYB*SZB + SXB*CZB;
	P2B[1][1] = -1.0*SXB*CYB*SZB + CXB*CZB;
	P2B[2][1] = -1.0*SYB*SZB;
	P2B[0][2] = CXB*SYB;
	P2B[1][2] = -1.0*SXB*SYB;
	P2B[2][2] = CYB;
}
